@@ -14,15 +14,14 @@
 //! terminates, as the result value of the rule will itself be Undefined.
 
 use crate::common;
-use crate::error::Error;
 
 /// Gets the next character
 /// # Arguments
 /// * `s`: String label
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
 /// The character immediately following the one at the offset position in logical
-/// order in the string representing the label. After(LastChar) evaluates to None.
+/// order in the string representing the label. After(`LastChar`) evaluates to None.
 #[inline]
 fn after(s: &str, offset: usize) -> Option<char> {
     s.chars().nth(offset + 1)
@@ -31,10 +30,10 @@ fn after(s: &str, offset: usize) -> Option<char> {
 /// Gets the previous character
 /// # Arguments
 /// * `s`: String label
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
 /// The character immediately preceding the one at the offset position in logical
-/// order in the string representing the label. Before(FirstChar) evaluates to None.
+/// order in the string representing the label. Before(`FirstChar`) evaluates to None.
 #[inline]
 fn before(s: &str, offset: usize) -> Option<char> {
     if offset == 0 {
@@ -44,131 +43,144 @@ fn before(s: &str, offset: usize) -> Option<char> {
     }
 }
 
-/// [Appendinx A.1](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.1).
-/// ZERO WIDTH NON-JOINER (U+200C)\
+/// Error associated to the application of any context rule.
+#[derive(Debug, PartialEq)]
+pub enum ContextRuleError {
+    /// Context rule is not applicable
+    NotApplicable,
+    /// Special value used to deal with any error conditions, such as an attempt to test
+    /// a character before the start of a label or after the end of a label.
+    Undefined,
+}
+
+/// [Appendix A.1](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.1).
+/// ZERO WIDTH NON-JOINER `U+200C`
 /// This may occur in a formally cursive script (such as Arabic) in a
 /// context where it breaks a cursive connection as required for
 /// orthographic rules, as in the Persian language, for example.  It
-/// also may occur in Indic scripts in a consonant-conjunct context
-/// (immediately following a virama), to control required display of
+/// also may occur in `Indic` scripts in a consonant-conjunct context
+/// (immediately following a `virama`), to control required display of
 /// such conjuncts.
 /// # Arguments
 /// * `s`: String value to check
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
-/// True if context permits a ZERO WIDTH NON-JOINER (U+200C).
-pub fn rule_zero_width_nonjoiner(s: &str, offset: usize) -> Result<bool, Error> {
-    if 0x200c != s.chars().nth(offset).ok_or(Error::Undefined)? as u32 {
-        return Err(Error::NotApplicable);
+/// True if context permits a ZERO WIDTH NON-JOINER `U+200C`.
+pub fn rule_zero_width_nonjoiner(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    if 0x200c != s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32 {
+        return Err(ContextRuleError::NotApplicable);
     }
 
-    let mut prev = before(s, offset).ok_or(Error::Undefined)?;
+    let mut prev = before(s, offset).ok_or(ContextRuleError::Undefined)?;
     let mut cp = prev as u32;
     if common::is_virama(cp) {
         return Ok(true);
     }
 
-    // RegExpMatch((Joining_Type:{L,D})(Joining_Type:T)*\u200C
-    //     (Joining_Type:T)*(Joining_Type:{R,D}))
+    // `RegExpMatch`((`Joining_Type`:`{L,D}`)(`Joining_Type`:T)*`U+200C`
+    //     (`Joining_Type`:T)*(`Joining_Type`:`{R,D}`))
 
-    // Check all transparent joining type codepoints before {u200C} (0 or more)
+    // Check all transparent joining type code points before `U+200C` (0 or more)
     let mut i = offset - 1;
     while common::is_transparent(cp) {
-        prev = before(s, i).ok_or(Error::Undefined)?;
+        prev = before(s, i).ok_or(ContextRuleError::Undefined)?;
         cp = prev as u32;
         i -= 1;
     }
 
-    // Joining_Type:{L,D}
+    // `Joining_Type`:`{L,D}`
     if !(common::is_left_joining(cp) || common::is_dual_joining(cp)) {
         return Ok(false);
     }
 
-    // Check all transparent joining type codepoints following {u200C} (0 or more)
-    let mut next = after(s, offset).ok_or(Error::Undefined)?;
+    // Check all transparent joining type code points following `U+200C` (0 or more)
+    let mut next = after(s, offset).ok_or(ContextRuleError::Undefined)?;
     cp = next as u32;
     i = offset + 1;
     while common::is_transparent(cp) {
-        next = after(s, i).ok_or(Error::Undefined)?;
+        next = after(s, i).ok_or(ContextRuleError::Undefined)?;
         cp = next as u32;
         i += 1;
     }
 
-    // Joining_Type:{R,D}
+    // `Joining_Type`:`{R,D}`
     Ok(common::is_right_joining(cp) || common::is_dual_joining(cp))
 }
 
-/// [Appendinx A.2](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.2).
+/// [Appendix A.2](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.2).
 /// ZERO WIDTH JOINER\
-/// This may occur in Indic scripts in a consonant-conjunct context
-/// (immediately following a virama), to control required display of
+/// This may occur in `Indic` scripts in a consonant-conjunct context
+/// (immediately following a `virama`), to control required display of
 /// such conjuncts.
 /// # Arguments
 /// * `s`: String value to check
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
-/// Return true if context permits a ZERO WIDTH JOINER (U+200D).
-pub fn rule_zero_width_joiner(s: &str, offset: usize) -> Result<bool, Error> {
-    if 0x200d != s.chars().nth(offset).ok_or(Error::Undefined)? as u32 {
-        return Err(Error::NotApplicable);
+/// Return true if context permits a ZERO WIDTH JOINER `U+200D`.
+pub fn rule_zero_width_joiner(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    if 0x200d != s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32 {
+        return Err(ContextRuleError::NotApplicable);
     }
-    let prev = before(s, offset).ok_or(Error::Undefined)?;
+    let prev = before(s, offset).ok_or(ContextRuleError::Undefined)?;
     Ok(common::is_virama(prev as u32))
 }
 
-/// [Appendinx A.3](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.3).
+/// [Appendix A.3](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.3).
 /// MIDDLE DOT\
-/// Between 'l' (U+006C) characters only, used to permit the Catalan
-/// character ela geminada to be expressed.
+/// Between 'l' `U+006C` characters only, used to permit the Catalan
+/// character `ela` `geminada` to be expressed.
 /// # Arguments
 /// * `s`: String value to check
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
-/// Return true if context permits a MIDDLE DOT (U+00B7).
-pub fn rule_middle_dot(s: &str, offset: usize) -> Result<bool, Error> {
-    if 0x00b7 != s.chars().nth(offset).ok_or(Error::Undefined)? as u32 {
-        return Err(Error::NotApplicable);
+/// Return true if context permits a MIDDLE DOT `U+00B7`.
+pub fn rule_middle_dot(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    if 0x00b7 != s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32 {
+        return Err(ContextRuleError::NotApplicable);
     }
-    let prev = before(s, offset).ok_or(Error::Undefined)?;
-    let next = after(s, offset).ok_or(Error::Undefined)?;
+    let prev = before(s, offset).ok_or(ContextRuleError::Undefined)?;
+    let next = after(s, offset).ok_or(ContextRuleError::Undefined)?;
     Ok(prev as u32 == 0x006c && next as u32 == 0x006c)
 }
 
-/// [Appendinx A.4](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.4).
-/// GREEK LOWER NUMERAL SIGN (KERAIA)\
+/// [Appendix A.4](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.4).
+/// GREEK LOWER NUMERAL SIGN (`KERAIA`)\
 /// The script of the following character MUST be Greek.
 /// # Arguments
 /// * `s`: String value to check
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
-/// Return true if context permits GREEK LOWER NUMERAL SIGN (U+0375).
-pub fn rule_greek_lower_numeral_sign_keraia(s: &str, offset: usize) -> Result<bool, Error> {
-    if 0x0375 != s.chars().nth(offset).ok_or(Error::Undefined)? as u32 {
-        return Err(Error::NotApplicable);
+/// Return true if context permits GREEK LOWER NUMERAL SIGN `U+0375`.
+pub fn rule_greek_lower_numeral_sign_keraia(
+    s: &str,
+    offset: usize,
+) -> Result<bool, ContextRuleError> {
+    if 0x0375 != s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32 {
+        return Err(ContextRuleError::NotApplicable);
     }
-    let after = after(s, offset).ok_or(Error::Undefined)?;
+    let after = after(s, offset).ok_or(ContextRuleError::Undefined)?;
     Ok(common::is_greek(after as u32))
 }
 
-/// [Appendinx A.5](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.5).
-/// [Appendinx A.6](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.5).
-/// HEBREW PUNCTUATION GERESH and HEBREW PUNCTUATION GERSHAYIM\
+/// [Appendix A.5](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.5).
+/// [Appendix A.6](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.5).
+/// HEBREW PUNCTUATION `GERESH` and HEBREW PUNCTUATION `GERSHAYIM`\
 /// The script of the preceding character MUST be Hebrew.
 /// # Arguments
 /// * `s`: String value to check
-/// * `offset`: The positoin of the character in the label
+/// * `offset`: The position of the character in the label
 /// # Returns
-/// Return true if context permits HEBREW PUNCTUATION GERESH or GERSHAYIM (U+05F3, U+05F4).
-pub fn rule_hebrew_punctuation(s: &str, offset: usize) -> Result<bool, Error> {
-    let cp = s.chars().nth(offset).ok_or(Error::Undefined)? as u32;
+/// Return true if context permits HEBREW PUNCTUATION `GERESH` or `GERSHAYIM` (`U+05F3`, `U+05F4`).
+pub fn rule_hebrew_punctuation(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    let cp = s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32;
     if cp != 0x05f3 && cp != 0x05f4 {
-        return Err(Error::NotApplicable);
+        return Err(ContextRuleError::NotApplicable);
     }
-    let prev = before(s, offset).ok_or(Error::Undefined)?;
+    let prev = before(s, offset).ok_or(ContextRuleError::Undefined)?;
     Ok(common::is_hebrew(prev as u32))
 }
 
-/// [Appendinx A.7](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.7).
+/// [Appendix A.7](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.7).
 /// KATAKANA MIDDLE DOT\
 /// Note that the Script of Katakana Middle Dot is not any of
 /// "Hiragana", "Katakana", or "Han".  The effect of this rule is to
@@ -177,10 +189,10 @@ pub fn rule_hebrew_punctuation(s: &str, offset: usize) -> Result<bool, Error> {
 /// # Arguments
 /// * `s`: String value to check
 /// # Returns
-/// Return true if context permits KATAKANA MIDDLE DOT (U+30FB).
-pub fn rule_katakana_middle_dot(s: &str, offset: usize) -> Result<bool, Error> {
-    if 0x30fb != s.chars().nth(offset).ok_or(Error::Undefined)? as u32 {
-        return Err(Error::NotApplicable);
+/// Return true if context permits KATAKANA MIDDLE DOT `U+30FB`.
+pub fn rule_katakana_middle_dot(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    if 0x30fb != s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32 {
+        return Err(ContextRuleError::NotApplicable);
     }
     for c in s.chars() {
         let cp = c as u32;
@@ -192,17 +204,17 @@ pub fn rule_katakana_middle_dot(s: &str, offset: usize) -> Result<bool, Error> {
     Ok(false)
 }
 
-/// [Appendinx A.8](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.8).
+/// [Appendix A.8](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.8).
 /// ARABIC-INDIC DIGITS\
 /// Can not be mixed with Extended Arabic-Indic Digits.
 /// # Arguments
 /// * `s`: String value to check
 /// # Returns
-/// Return true if context permits ARABIC-INDIC DIGITS (U+0660..U+0669).
-pub fn rule_arabic_indic_digits(s: &str, offset: usize) -> Result<bool, Error> {
-    let cp = s.chars().nth(offset).ok_or(Error::Undefined)? as u32;
+/// Return true if context permits ARABIC-INDIC DIGITS (`U+0660`..`U+0669`).
+pub fn rule_arabic_indic_digits(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    let cp = s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32;
     if !(0x0660..=0x0669).contains(&cp) {
-        return Err(Error::NotApplicable);
+        return Err(ContextRuleError::NotApplicable);
     }
     let range = 0x06f0..=0x06f9;
     for c in s.chars() {
@@ -214,17 +226,17 @@ pub fn rule_arabic_indic_digits(s: &str, offset: usize) -> Result<bool, Error> {
     Ok(true)
 }
 
-/// [Appendinx A.9](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.9).
+/// [Appendix A.9](https://datatracker.ietf.org/doc/html/rfc5892#appendix-A.9).
 /// EXTENDED ARABIC-INDIC DIGITS\
 /// Can not be mixed with Arabic-Indic Digits.
 /// # Arguments
 /// * `s`: String value to check
 /// # Returns
-/// Return true if context permits EXTENDED ARABIC-INDIC DIGITS (U+06F0..U+06F9).
-pub fn rule_extended_arabic_indic_digits(s: &str, offset: usize) -> Result<bool, Error> {
-    let cp = s.chars().nth(offset).ok_or(Error::Undefined)? as u32;
+/// Return true if context permits EXTENDED ARABIC-INDIC DIGITS (`U+06F0`..`U+06F9`).
+pub fn rule_extended_arabic_indic_digits(s: &str, offset: usize) -> Result<bool, ContextRuleError> {
+    let cp = s.chars().nth(offset).ok_or(ContextRuleError::Undefined)? as u32;
     if !(0x06f0..=0x06f9).contains(&cp) {
-        return Err(Error::NotApplicable);
+        return Err(ContextRuleError::NotApplicable);
     }
     let range = 0x0660..=0x0669;
     for c in s.chars() {
@@ -237,14 +249,14 @@ pub fn rule_extended_arabic_indic_digits(s: &str, offset: usize) -> Result<bool,
 }
 
 /// Describes a context rule function
-pub type ContextRule = fn(s: &str, offset: usize) -> Result<bool, Error>;
+pub type ContextRule = fn(s: &str, offset: usize) -> Result<bool, ContextRuleError>;
 
 /// Gets the context rule associated to an Unicode code point.
 /// Arguments
 /// * `cp`: Unicode code point
 /// # Returns
 /// The context rule function or None if there is no context rule
-/// defined for the codepoint `cp`
+/// defined for the code point `cp`
 pub fn get_context_rule(cp: u32) -> Option<ContextRule> {
     match cp {
         0x00b7 => Some(rule_middle_dot),
@@ -287,111 +299,111 @@ mod tests {
 
     #[test]
     fn check_rule_zero_width_nonjoiner() {
-        // code point at position 0 is not 0x200c
+        // code point at position 0 is not `U+200C`
         let label = "A";
         let res = rule_zero_width_nonjoiner(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // Before(FirstChar) evaluates to Undefined.
+        // Before(`FirstChar`) evaluates to Undefined.
         let label = "\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // Before(cp) == Virama => true
+        // Before(`cp`) equal to `Virama` then true
         let label = "\u{94d}\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Before(cp) == Virama => true
+        // Before(`cp`) equal to `Virama` then true
         let label = "A\u{94d}\u{200c}B";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Previous cp is neither Virama nor transparent/Joining_Type:{L,D} => false
+        // Previous `cp` is neither `Virama` nor transparent/`Joining_Type`:`{L,D}` then false
         let label = "A\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // Miss Joining_Type:{L,D} before Transparent => Error(Undefined)
-        // "(Joining_Type:T)\u200C"
+        // Miss `Joining_Type`:`{L,D}` before Transparent then undefined error
+        // "(`Joining_Type`:T)`U+200C`"
         let label = "\u{5bf}\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 1);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // No Joining_Type:{L,D} before Transparent => false
-        // 'A'(Joining_Type:T)\u200C
+        // No `Joining_Type`:`{L,D}` before Transparent then false
+        // 'A'(`Joining_Type`:T)`U+200C`
         let label = "A\u{5bf}\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // First part of the regExp is complete but fails to meet the second one
-        // (Joining_Type:L)(Joining_Type:T)\u200C
+        // First part of the `regExp` is complete but fails to meet the second one
+        // (`Joining_Type`:L)(`Joining_Type`:T)`U+200C`
         let label = "\u{a872}\u{5bf}\u{200c}";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // First part of the regExp is complete but fails to meet the second one
-        // (Joining_Type:L)(Joining_Type:T)\u200C(Joining_Type:T)
+        // First part of the `regExp` is complete but fails to meet the second one
+        // (`Joining_Type`:L)(`Joining_Type`:T)`U+200C`(`Joining_Type`:T)
         let label = "\u{a872}\u{5bf}\u{200c}\u{5bf}";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // Label matches RegExp
-        // (Joining_Type:L)(Joining_Type:T)\u200C(Joining_Type:T)(Joining_Type:R)
+        // Label matches `RegExp`
+        // (`Joining_Type`:L)(`Joining_Type`:T)`U+200C`(`Joining_Type`:T)(`Joining_Type`:R)
         let label = "\u{a872}\u{5bf}\u{200c}\u{5bf}\u{629}";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Label does not match RegExp
-        // (Joining_Type:L)(Joining_Type:T)\u200C(Joining_Type:T)'A'
+        // Label does not match `RegExp`
+        // (`Joining_Type`:L)(`Joining_Type`:T)`U+200C`(`Joining_Type`:T)'A'
         let label = "\u{a872}\u{5bf}\u{200c}\u{5bf}A";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // Label does not matches RegExp
-        // (Joining_Type:L)(Joining_Type:T)\u200C'A'
+        // Label does not matches `RegExp`
+        // (`Joining_Type`:L)(`Joining_Type`:T)`U+200C`'A'
         let label = "\u{a872}\u{5bf}\u{200c}A";
         let res = rule_zero_width_nonjoiner(label, 2);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // 'A'(Joining_Type:T){2}\u200C(Joining_Type:T){4}(Joining_Type:D)
+        // 'A'(`Joining_Type`:T)(2)`U+200C`(`Joining_Type`:T)(4)(`Joining_Type`:D)
         let label = "A\u{5bf}\u{5bf}\u{200c}\u{5bf}\u{5bf}\u{5bf}\u{5bf}\u{626}";
         let res = rule_zero_width_nonjoiner(label, 3);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // All next tests should match RegExp
+        // All next tests should match `RegExp`
 
-        // (Joining_Type:D)\u200C(Joining_Type:T)(Joining_Type:D)
+        // (`Joining_Type`:D)`U+200C`(`Joining_Type`:T)(`Joining_Type`:D)
         let label = "\u{626}\u{200c}\u{5bf}\u{626}";
         let res = rule_zero_width_nonjoiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // (Joining_Type:D)\u200C(Joining_Type:D)
+        // (`Joining_Type`:D)`U+200C`(`Joining_Type`:D)
         let label = "\u{626}\u{200c}\u{626}";
         let res = rule_zero_width_nonjoiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // (Joining_Type:D)(Joining_Type:T){2}\u200C(Joining_Type:T){4}(Joining_Type:D)
+        // (`Joining_Type`:D)(`Joining_Type`:T)(2)`U+200C`(`Joining_Type`:T)(4)(`Joining_Type`:D)
         let label = "\u{626}\u{5bf}\u{5bf}\u{200c}\u{5bf}\u{5bf}\u{5bf}\u{5bf}\u{626}";
         let res = rule_zero_width_nonjoiner(label, 3);
         assert_eq!(res.is_ok(), true);
@@ -403,36 +415,36 @@ mod tests {
         let label = "";
         let res = rule_zero_width_joiner(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "A";
         let res = rule_zero_width_joiner(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{200d}";
         let res = rule_zero_width_joiner(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "\u{200d}A";
         let res = rule_zero_width_joiner(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // Canonical_Combining_Class(Before(cp)) .eq.  Virama Then True
+        // `Canonical_Combining_Class`(Before(`cp`)) .`eq`.  `Virama` Then True
         let label = "\u{94d}\u{200d}";
         let res = rule_zero_width_joiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Canonical_Combining_Class(Before(cp)) .ne.  Virama Then False
+        // `Canonical_Combining_Class`(Before(`cp`)) .`ne`.  `Virama` Then False
         let label = "A\u{200d}";
         let res = rule_zero_width_joiner(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // Canonical_Combining_Class(Before(cp)) .eq.  Virama Then True
+        // `Canonical_Combining_Class`(Before(`cp`)) .`eq`.  `Virama` Then True
         let label = "A\u{94d}\u{200d}B";
         let res = rule_zero_width_joiner(label, 2);
         assert_eq!(res.is_ok(), true);
@@ -444,22 +456,22 @@ mod tests {
         let label = "";
         let res = rule_middle_dot(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "A";
         let res = rule_middle_dot(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{00b7}";
         let res = rule_middle_dot(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "\u{006c}\u{00b7}";
         let res = rule_middle_dot(label, 1);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "\u{006c}\u{00b7}\u{006c}";
         let res = rule_middle_dot(label, 1);
@@ -482,19 +494,19 @@ mod tests {
         let label = "";
         let res = rule_greek_lower_numeral_sign_keraia(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "A";
         let res = rule_greek_lower_numeral_sign_keraia(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{0375}";
         let res = rule_greek_lower_numeral_sign_keraia(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // Script(After(cp)) .eq.  Greek Then True
+        // Script(After(`cp`)) .`eq`.  Greek Then True
         let label = "\u{0375}\u{0384}";
         let res = rule_greek_lower_numeral_sign_keraia(label, 0);
         assert_eq!(res.is_ok(), true);
@@ -505,7 +517,7 @@ mod tests {
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Script(After(cp)) .ne.  Greek Then False
+        // Script(After(`cp`)) .`ne`.  Greek Then False
         let label = "\u{0375}A";
         let res = rule_greek_lower_numeral_sign_keraia(label, 0);
         assert_eq!(res.is_ok(), true);
@@ -517,37 +529,37 @@ mod tests {
         let label = "";
         let res = rule_hebrew_punctuation(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "A";
         let res = rule_hebrew_punctuation(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{05F3}";
         let res = rule_hebrew_punctuation(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
-        // [GERESH] Script(Before(cp)) .eq.  Hebrew Then True;
+        // [`GERESH`] Script(Before(`cp`)) .`eq`.  Hebrew Then True;
         let label = "\u{5f0}\u{05F3}";
         let res = rule_hebrew_punctuation(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // [GERSHAYIM] Script(Before(cp)) .eq.  Hebrew Then True;
+        // [`GERSHAYIM`] Script(Before(`cp`)) .`eq`.  Hebrew Then True;
         let label = "\u{5f0}\u{05F4}";
         let res = rule_hebrew_punctuation(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Script(Before(cp)) .ne.  Hebrew Then False;
+        // Script(Before(`cp`)) .`ne`.  Hebrew Then False;
         let label = "A\u{05F4}";
         let res = rule_hebrew_punctuation(label, 1);
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), false);
 
-        // [GERSHAYIM] Script(Before(cp)) .eq.  Hebrew Then True;
+        // [`GERSHAYIM`] Script(Before(`cp`)) .`eq`.  Hebrew Then True;
         let label = "YYY\u{5f0}\u{05F4}XXX";
         let res = rule_hebrew_punctuation(label, 4);
         assert_eq!(res.is_ok(), true);
@@ -559,12 +571,12 @@ mod tests {
         let label = "";
         let res = rule_katakana_middle_dot(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "A";
         let res = rule_katakana_middle_dot(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{30fb}";
         let res = rule_katakana_middle_dot(label, 0);
@@ -600,19 +612,19 @@ mod tests {
         let label = "";
         let res = rule_arabic_indic_digits(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "\u{065f}";
         let res = rule_arabic_indic_digits(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{066a}";
         let res = rule_arabic_indic_digits(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
-        // Check values in range [0x0660..0x0669]
+        // Check values in range [`0x0660`..`0x0669`]
         let label = "\u{0660}";
         let res = rule_arabic_indic_digits(label, 0);
         assert_eq!(res.is_ok(), true);
@@ -628,7 +640,7 @@ mod tests {
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Label does not contain Extended Arabic-Indic Digits => True
+        // Label does not contain Extended Arabic-Indic Digits then True
         let label = "ab\u{0669}cd";
         let res = rule_arabic_indic_digits(label, 2);
         assert_eq!(res.is_ok(), true);
@@ -644,7 +656,7 @@ mod tests {
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Label contains Extended Arabic-Indic Digits => False
+        // Label contains Extended Arabic-Indic Digits then False
         let label = "ab\u{0669}c\u{06f0}";
         let res = rule_arabic_indic_digits(label, 2);
         assert_eq!(res.is_ok(), true);
@@ -661,19 +673,19 @@ mod tests {
         let label = "";
         let res = rule_extended_arabic_indic_digits(label, 3);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::Undefined);
+        assert_eq!(res.unwrap_err(), ContextRuleError::Undefined);
 
         let label = "\u{06ef}";
         let res = rule_extended_arabic_indic_digits(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
         let label = "\u{06fa}";
         let res = rule_extended_arabic_indic_digits(label, 0);
         assert_eq!(res.is_err(), true);
-        assert_eq!(res.unwrap_err(), Error::NotApplicable);
+        assert_eq!(res.unwrap_err(), ContextRuleError::NotApplicable);
 
-        // Check values in range [0x06f0..0x06f9]
+        // Check values in range [`0x06f0`..`0x06f9`]
         let label = "\u{06f0}";
         let res = rule_extended_arabic_indic_digits(label, 0);
         assert_eq!(res.is_ok(), true);
@@ -689,7 +701,7 @@ mod tests {
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Label does not contain Arabic-Indic Digits => True
+        // Label does not contain Arabic-Indic Digits then True
         let label = "ab\u{06f0}cd";
         let res = rule_extended_arabic_indic_digits(label, 2);
         assert_eq!(res.is_ok(), true);
@@ -705,7 +717,7 @@ mod tests {
         assert_eq!(res.is_ok(), true);
         assert_eq!(res.unwrap(), true);
 
-        // Label contains Extended Arabic-Indic Digits => False
+        // Label contains Extended Arabic-Indic Digits then False
         let label = "ab\u{06f0}c\u{0660}";
         let res = rule_extended_arabic_indic_digits(label, 2);
         assert_eq!(res.is_ok(), true);
