@@ -1,26 +1,31 @@
+//! This module contains the implementation and traits for the
+//! String classes such as it is defined by the PRECIS framework
+//! [`rfc8264`](https://datatracker.ietf.org/doc/html/rfc8264#section-4)
+
 use crate::common;
 use crate::context;
 use crate::DerivedPropertyValue;
+use crate::{CodepointInfo, Error, UnexpectedError};
 
-/// Interface for specific classes to deal with specific unicode
+/// Interface for specific classes to deal with specific Unicode
 /// code groups defined in RFC 8264.
 /// Next callbacks will be invoked to calculate the derived property
-/// value in the order defined in [`get_derived_property_value`]
+/// according to the algorithm defined in [`rfc8264`](https://datatracker.ietf.org/doc/html/rfc8264#section-8)
 pub trait SpecificDerivedPropertyValue {
-    /// Callback invoked when the unicode code point belongs to
+    /// Callback invoked when the Unicode code point belongs to
     /// [Spaces](https://datatracker.ietf.org/doc/html/rfc8264#section-9.14)
     fn on_spaces(&self) -> DerivedPropertyValue;
-    /// Callback invoked when the unicode code point belongs to
+    /// Callback invoked when the Unicode code point belongs to
     /// [Symbols](https://datatracker.ietf.org/doc/html/rfc8264#section-9.15)
     fn on_symbols(&self) -> DerivedPropertyValue;
-    /// Callback invoked when the unicode code point belongs to
+    /// Callback invoked when the Unicode code point belongs to
     /// [Punctuation](https://datatracker.ietf.org/doc/html/rfc8264#section-9.16)
     fn on_punctuation(&self) -> DerivedPropertyValue;
-    /// Callback invoked when the unicode code point belongs to
-    /// [HasCompat](https://datatracker.ietf.org/doc/html/rfc8264#section-9.17)
+    /// Callback invoked when the Unicode code point belongs to
+    /// [`HasCompat`](https://datatracker.ietf.org/doc/html/rfc8264#section-9.17)
     fn on_has_compat(&self) -> DerivedPropertyValue;
-    /// Callback invoked when the unicode code point belongs to
-    /// [OtherLetterDigits](https://datatracker.ietf.org/doc/html/rfc8264#section-9.18)
+    /// Callback invoked when the Unicode code point belongs to
+    /// [`OtherLetterDigits`](https://datatracker.ietf.org/doc/html/rfc8264#section-9.18)
     fn on_other_letter_digits(&self) -> DerivedPropertyValue;
 }
 
@@ -29,31 +34,31 @@ pub trait SpecificDerivedPropertyValue {
 /// of operations within this algorithm, because doing so would cause
 /// inconsistent results across implementations):
 ///
-/// > If .cp. .in. Exceptions Then Exceptions(cp);\
-/// > Else If .cp. .in. BackwardCompatible Then BackwardCompatible(cp);\
-/// > Else If .cp. .in. Unassigned Then UNASSIGNED;\
-/// > Else If .cp. .in. ASCII7 Then PVALID;\
-/// > Else If .cp. .in. JoinControl Then CONTEXTJ;\
-/// > Else If .cp. .in. OldHangulJamo Then DISALLOWED;\
-/// > Else If .cp. .in. PrecisIgnorableProperties Then DISALLOWED;\
-/// > Else If .cp. .in. Controls Then DISALLOWED;\
-/// > Else If .cp. .in. HasCompat Then ID_DIS or FREE_PVAL;\
-/// > Else If .cp. .in. LetterDigits Then PVALID;\
-/// > Else If .cp. .in. OtherLetterDigits Then ID_DIS or FREE_PVAL;\
-/// > Else If .cp. .in. Spaces Then ID_DIS or FREE_PVAL;\
-/// > Else If .cp. .in. Symbols Then ID_DIS or FREE_PVAL;\
-/// > Else If .cp. .in. Punctuation Then ID_DIS or FREE_PVAL;\
+/// > If .`cp`. .in. `Exceptions` Then `Exceptions`(`cp`);\
+/// > Else If .`cp`. .in. `BackwardCompatible` Then `BackwardCompatible`(`cp`);\
+/// > Else If .`cp`. .in. `Unassigned` Then `UNASSIGNED`;\
+/// > Else If .`cp`. .in. `ASCII7` Then `PVALID`;\
+/// > Else If .`cp`. .in. `JoinControl` Then `CONTEXTJ`;\
+/// > Else If .`cp`. .in. `OldHangulJamo` Then `DISALLOWED`;\
+/// > Else If .`cp`. .in. `PrecisIgnorableProperties` Then `DISALLOWED`;\
+/// > Else If .`cp`. .in. `Controls` Then `DISALLOWED`;\
+/// > Else If .`cp`. .in. `HasCompat` Then `ID_DIS` or `FREE_PVAL`;\
+/// > Else If .`cp`. .in. `LetterDigits` Then `PVALID`;\
+/// > Else If .`cp`. .in. `OtherLetterDigits` Then `ID_DIS` or `FREE_PVAL`;\
+/// > Else If .`cp`. .in. `Spaces` Then `ID_DIS` or `FREE_PVAL`;\
+/// > Else If .`cp`. .in. `Symbols` Then `ID_DIS` or `FREE_PVAL`;\
+/// > Else If .`cp`. .in. `Punctuation` Then `ID_DIS` or `FREE_PVAL`;\
 /// > Else DISALLOWED;\
 ///
 /// # Arguments
 /// * `cp` - Unicode code point
-/// * `obj` - Object implementing the SpecificDerivedPropertyValue trait.
+/// * `obj` - Object implementing the [`SpecificDerivedPropertyValue`] trait.
 ///
 /// # Return
 /// This function returns the derived property value as defined in
 /// [RFC 8264](https://datatracker.ietf.org/doc/html/rfc8264#section-8)
 #[allow(clippy::if_same_then_else)]
-pub fn get_derived_property_value(
+fn get_derived_property_value(
     cp: u32,
     obj: &dyn SpecificDerivedPropertyValue,
 ) -> DerivedPropertyValue {
@@ -94,41 +99,52 @@ pub fn get_derived_property_value(
     }
 }
 
-fn allowed_by_context_rule(s: &str, cp: u32, offset: usize) -> bool {
-    let val = context::get_context_rule(cp);
-
-    debug_assert!(
-        val.is_some(),
-        "No context rule found for Unicode code point: {:#04x}",
-        cp
-    );
-
-    let mut allowed = false;
-    if let Some(rule) = val {
-        if let Ok(ret) = rule(s, offset) {
-            allowed = ret;
-        };
+fn allowed_by_context_rule(
+    label: &str,
+    val: DerivedPropertyValue,
+    cp: u32,
+    offset: usize,
+) -> Result<(), Error> {
+    match context::get_context_rule(cp) {
+        None => Err(Error::Unexpected(UnexpectedError::MissingContextRule(
+            CodepointInfo::new(cp, offset, val),
+        ))),
+        Some(rule) => match rule(label, offset) {
+            Ok(allowed) => {
+                if allowed {
+                    Ok(())
+                } else {
+                    Err(Error::BadCodepoint(CodepointInfo::new(cp, offset, val)))
+                }
+            }
+            Err(e) => match e {
+                context::ContextRuleError::NotApplicable => Err(Error::Unexpected(
+                    UnexpectedError::ContextRuleNotApplicable(CodepointInfo::new(cp, offset, val)),
+                )),
+                context::ContextRuleError::Undefined => {
+                    Err(Error::Unexpected(UnexpectedError::Undefined))
+                }
+            },
+        },
     }
-
-    allowed
 }
 
 /// Base interface for all String classes in PRECIS framework.
 pub trait StringClass {
     /// Gets the derived property value according to the algorithm defined
-    /// in [``get_derived_property_value`]
+    /// in [`rfc8264`](https://datatracker.ietf.org/doc/html/rfc8264#section-8)
     /// # Arguments
-    /// * `c`- unicode character
+    /// * `c`- Unicode character
     /// # Return
-    /// This method returns the derived property value associated to a unicode character
+    /// This method returns the derived property value associated to a Unicode character
     fn get_value_from_char(&self, c: char) -> DerivedPropertyValue;
 
     /// Gets the derived property value according to the algorithm defined
-    /// in [``get_derived_property_value`]
+    /// in [`rfc8264`](https://datatracker.ietf.org/doc/html/rfc8264#section-8)
     /// # Arguments:
-    /// * `cp`- unicode codepoint
+    /// * `cp`- Unicode code point
     /// # Return
-    /// This method returns the derived property value associated to a unicode character
+    /// This method returns the derived property value associated to a Unicode character
     fn get_value_from_codepoint(&self, cp: u32) -> DerivedPropertyValue;
 
     /// Ensures that the string consists only of Unicode code points that
@@ -138,28 +154,28 @@ pub trait StringClass {
     /// * `label` - string to check
     /// # Returns
     /// true if all character of `label` are allowed by the String Class.
-    fn allows(&self, label: &str) -> bool {
+    fn allows(&self, label: &str) -> Result<(), Error> {
         for (offset, c) in label.chars().enumerate() {
             let val = self.get_value_from_char(c);
 
             match val {
-                DerivedPropertyValue::PValid | DerivedPropertyValue::SpecClassPval => {}
+                DerivedPropertyValue::PValid | DerivedPropertyValue::SpecClassPval => Ok(()),
                 DerivedPropertyValue::SpecClassDis
                 | DerivedPropertyValue::Disallowed
-                | DerivedPropertyValue::Unassigned => return false,
+                | DerivedPropertyValue::Unassigned => Err(Error::BadCodepoint(CodepointInfo::new(
+                    c as u32, offset, val,
+                ))),
                 DerivedPropertyValue::ContextJ | DerivedPropertyValue::ContextO => {
-                    if !allowed_by_context_rule(label, c as u32, offset) {
-                        return false;
-                    }
+                    allowed_by_context_rule(label, val, c as u32, offset)
                 }
-            }
+            }?
         }
 
-        true
+        Ok(())
     }
 }
 
-/// Concrete class representing PRECIS IdentifierClass from
+/// Concrete class representing PRECIS `IdentifierClass` from
 /// [RFC 8264](https://datatracker.ietf.org/doc/html/rfc8264#section-4.2).
 /// # Example
 /// ```rust
@@ -176,7 +192,7 @@ pub trait StringClass {
 pub struct IdentifierClass {}
 
 impl SpecificDerivedPropertyValue for IdentifierClass {
-    // ID_DIS mapped to SPEC_CLASS_DIS
+    // `ID_DIS` mapped to `SPEC_CLASS_DIS`
     fn on_has_compat(&self) -> DerivedPropertyValue {
         DerivedPropertyValue::SpecClassDis
     }
@@ -204,7 +220,7 @@ impl StringClass for IdentifierClass {
     }
 }
 
-/// Concrete class representing PRECIS FreeformClass from
+/// Concrete class representing PRECIS `FreeformClass` from
 /// [RFC 8264](https://datatracker.ietf.org/doc/html/rfc8264#section-4.3).
 /// # Example
 /// ```rust
@@ -247,131 +263,3 @@ impl StringClass for FreeformClass {
         get_derived_property_value(cp, self)
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use crate::*;
-    use precis_tools::*;
-    use std::env;
-    use std::fs::File;
-    use std::path::Path;
-    use std::path::PathBuf;
-
-    fn validate_result(
-        cp: u32,
-        expected: precis_tools::DerivedProperty,
-        id: &IdentifierClass,
-        ff: &FreeformClass,
-    ) {
-        match expected {
-            precis_tools::DerivedProperty::PValid => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert_eq!(id_prop, DerivedPropertyValue::PValid);
-                assert_eq!(ff_prop, DerivedPropertyValue::PValid);
-            }
-            precis_tools::DerivedProperty::FreePVal => {
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert_eq!(ff_prop, DerivedPropertyValue::SpecClassPval)
-            }
-            precis_tools::DerivedProperty::ContextJ => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert_eq!(id_prop, DerivedPropertyValue::ContextJ);
-                assert_eq!(ff_prop, DerivedPropertyValue::ContextJ);
-            }
-            precis_tools::DerivedProperty::ContextO => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert_eq!(id_prop, DerivedPropertyValue::ContextO);
-                assert_eq!(ff_prop, DerivedPropertyValue::ContextO);
-            }
-            precis_tools::DerivedProperty::Disallowed => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert_eq!(id_prop, DerivedPropertyValue::Disallowed);
-                assert_eq!(ff_prop, DerivedPropertyValue::Disallowed);
-            }
-            precis_tools::DerivedProperty::IdDis => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                assert_eq!(id_prop, DerivedPropertyValue::SpecClassDis);
-            }
-            precis_tools::DerivedProperty::Unassigned => {
-                let id_prop = id.get_value_from_codepoint(cp);
-                let ff_prop = ff.get_value_from_codepoint(cp);
-
-                assert!(
-                    id_prop == DerivedPropertyValue::Unassigned,
-                    "failed check for unicode point: {:#06x}. Expected: {:?}, Got: {:?}",
-                    cp,
-                    expected,
-                    id_prop
-                );
-                assert!(
-                    ff_prop == DerivedPropertyValue::Unassigned,
-                    "failed check for unicode point: {:#06x}. Expected: {:?}, Got: {:?}",
-                    cp,
-                    expected,
-                    ff_prop
-                );
-            }
-        }
-    }
-
-    fn check_derived_property(
-        cp: u32,
-        props: &DerivedProperties,
-        id: &IdentifierClass,
-        ff: &FreeformClass,
-    ) {
-        match props {
-            precis_tools::DerivedProperties::Single(p) => validate_result(cp, *p, &id, &ff),
-            precis_tools::DerivedProperties::Tuple((p1, p2)) => {
-                validate_result(cp, *p1, &id, &ff);
-                validate_result(cp, *p2, &id, &ff);
-            }
-        }
-    }
-
-    #[cfg(feature = "networking")]
-    fn get_csv_path() -> PathBuf {
-        let out_dir = env::var_os("OUT_DIR").unwrap();
-        Path::new(&out_dir).join("csv/precis-tables-6.3.0.csv")
-    }
-
-    #[cfg(not(feature = "networking"))]
-    fn get_csv_path() -> PathBuf {
-        let base_dir = env::var_os("CARGO_MANIFEST_DIR").unwrap();
-        Path::new(&base_dir).join("resources/csv/precis-tables-6.3.0.csv")
-    }
-
-    #[test]
-    fn check_derived_properties() {
-        let id = IdentifierClass {};
-        let ff = FreeformClass {};
-
-        let csv_path = get_csv_path();
-
-        let parser: precis_tools::CsvLineParser<File, precis_tools::PrecisDerivedProperty> =
-            precis_tools::CsvLineParser::from_path(csv_path).unwrap();
-
-        for result in parser {
-            let prop = result.unwrap();
-            match prop.codepoints {
-                ucd_parse::Codepoints::Single(cp) => {
-                    check_derived_property(cp.value(), &prop.properties, &id, &ff)
-                }
-                ucd_parse::Codepoints::Range(r) => {
-                    for cp in r {
-                        check_derived_property(cp.value(), &prop.properties, &id, &ff)
-                    }
-                }
-            }
-        }
-    }
-}
